// ICON
//
// ---------------------------------------------------------------
// Copyright (C) 2004-2025, DWD, MPI-M, DKRZ, KIT, ETH, MeteoSwiss
// Contact information: icon-model.org
//
// See AUTHORS.TXT for a list of authors
// See LICENSES/ for license information
// SPDX-License-Identifier: BSD-3-Clause
// ---------------------------------------------------------------

#pragma once

#include <Kokkos_Core.hpp>
#include <types.hpp>

#define ICONMATH_DECLARE_RECON_LSQ_CELL_L(_type)                               \
  void recon_lsq_cell_l(                                                       \
      const _type *p_cc, const int *cell_neighbor_idx,                         \
      const int *cell_neighbor_blk, const _type *lsq_qtmat_c,                  \
      const _type *lsq_rmat_rdiag_c, const _type *lsq_rmat_utri_c,             \
      const _type *lsq_moments, _type *p_coeff, int i_startblk, int i_endblk,  \
      int i_startidx_in, int i_endidx_in, int slev, int elev, int nproma,      \
      bool l_consv, bool lacc, bool acc_async, int nblks_c, int nlev,          \
      int lsq_dim_unk, int lsq_dim_c)

#define ICONMATH_DECLARE_RECON_LSQ_CELL_L_SVD(_type)                           \
  void recon_lsq_cell_l_svd(                                                   \
      const _type *p_cc, const int *cell_neighbor_idx,                         \
      const int *cell_neighbor_blk, const _type *lsq_pseudoinv,                \
      const _type *lsq_moments, _type *p_coeff, int i_startblk, int i_endblk,  \
      int i_startidx_in, int i_endidx_in, int slev, int elev, int nproma,      \
      bool l_consv, bool lacc, bool acc_async, int nblks_c, int nlev,          \
      int lsq_dim_unk, int lsq_dim_c)

#define ICONMATH_DECLARE_RECON_LSQ_CELL_Q(_type)                               \
  void recon_lsq_cell_q(                                                       \
      const _type *p_cc, const int *lsq_idx_c, const int *lsq_blk_c,           \
      const _type *lsq_qtmat_c, const _type *lsq_rmat_rdiag_c,                 \
      const _type *lsq_rmat_utri_c, const _type *lsq_moments, _type *p_coeff,  \
      int i_startblk, int i_endblk, int i_startidx_in, int i_endidx_in,        \
      int slev, int elev, int nproma, int patch_id, bool l_limited_area,       \
      bool lacc, int nblks_c, int nlev, int lsq_dim_unk, int lsq_dim_c)

#define ICONMATH_DECLARE_RECON_LSQ_CELL_Q_SVD(_type)                           \
  void recon_lsq_cell_q_svd(                                                   \
      const _type *p_cc, const int *lsq_idx_c, const int *lsq_blk_c,           \
      const _type *lsq_pseudoinv, const _type *lsq_moments, _type *p_coeff,    \
      int i_startblk, int i_endblk, int i_startidx_in, int i_endidx_in,        \
      int slev, int elev, int nproma, int patch_id, bool l_limited_area,       \
      bool lacc, int nblks_c, int nlev, int lsq_dim_unk, int lsq_dim_c)

#define ICONMATH_DECLARE_RECON_LSQ_CELL_C(_type)                               \
  void recon_lsq_cell_c(                                                       \
      const _type *p_cc, const int *lsq_idx_c, const int *lsq_blk_c,           \
      const _type *lsq_qtmat_c, const _type *lsq_rmat_rdiag_c,                 \
      const _type *lsq_rmat_utri_c, const _type *lsq_moments, _type *p_coeff,  \
      int i_startblk, int i_endblk, int i_startidx_in, int i_endidx_in,        \
      int slev, int elev, int nproma, int patch_id, bool l_limited_area,       \
      bool lacc, int nblks_c, int nlev, int lsq_dim_unk, int lsq_dim_c)

#define ICONMATH_DECLARE_RECON_LSQ_CELL_C_SVD(_type)                           \
  void recon_lsq_cell_c_svd(                                                   \
      const _type *p_cc, const int *lsq_idx_c, const int *lsq_blk_c,           \
      const _type *lsq_pseudoinv, const _type *lsq_moments, _type *p_coeff,    \
      int i_startblk, int i_endblk, int i_startidx_in, int i_endidx_in,        \
      int slev, int elev, int nproma, int patch_id,                            \
      bool l_limited_area, bool lacc, int nblks_c, int nlev, int lsq_dim_unk,  \
      int lsq_dim_c)

#define ICONMATH_DECLARE_DIV3D(_type)                                          \
  void div3d(const _type *vec_e, const int *cell_edge_idx,                     \
             const int *cell_edge_blk, const _type *geofac_div,                \
             _type *div_vec_c, int i_startblk, int i_endblk,                   \
             int i_startidx_in, int i_endidx_in, int slev, int elev,           \
             int nproma, bool lacc, int nlev, int nblks_c, int nblks_e)

#define ICONMATH_DECLARE_DIV3D_2FIELD(_type)                                   \
  void div3d_2field(const _type *vec_e, const int *cell_edge_idx,              \
                    const int *cell_edge_blk, const _type *geofac_div,         \
                    _type *div_vec_c, const _type *in2, _type *out2,           \
                    int i_startblk, int i_endblk, int i_startidx_in,           \
                    int i_endidx_in, int slev, int elev, int nproma,           \
                    bool lacc, int nlev, int nblks_c, int nblks_e)

#define ICONMATH_DECLARE_DIV4D(_type)                                          \
  void div4d(const int *cell_edge_idx, const int *cell_edge_blk,               \
             const _type *geofac_div, const _type *f4din, _type *f4dout,       \
             int dim4d, int i_startblk, int i_endblk, int i_startidx_in,       \
             int i_endidx_in, const int *slev, const int *elev, int nproma,    \
             bool lacc, int nlev, int nblks_c, int nblks_e)

#define ICONMATH_DECLARE_DIV_AVG(_type)                                        \
  void div_avg(const _type *vec_e, const int *cell_neighbor_idx,               \
               const int *cell_neighbor_blk, const int *cell_edge_idx,         \
               const int *cell_edge_blk, const _type *geofac_div,              \
               const _type *avg_coeff, _type *div_vec_c, const _type *opt_in2, \
               _type *opt_out2, const int *i_startblk_in,                      \
               const int *i_endblk_in, const int *i_startidx_in,               \
               const int *i_endidx_in, int slev, int elev, int nproma,         \
               int patch_id, bool l_limited_area, bool l2fields, bool lacc,    \
               int nlev, int nblks_c, int nblks_e)

#define ICONMATH_DECLARE_ROT_VERTEX_ATMOS(_type)                               \
  void rot_vertex_atmos(                                                       \
      const _type *vec_e, const int *vert_edge_idx, const int *vert_edge_blk,  \
      const _type *geofac_rot, _type *rot_vec, int i_startblk, int i_endblk,   \
      int i_startidx_in, int i_endidx_in, int slev, int elev, int nproma,      \
      bool lacc, int nlev, int nblks_e, int nblks_v)

#define ICONMATH_DECLARE_ROT_VERTEX_RI(_type)                                  \
  void rot_vertex_ri(                                                          \
      const _type *vec_e, const int *vert_edge_idx, const int *vert_edge_blk,  \
      const _type *geofac_rot, _type *rot_vec, int i_startblk, int i_endblk,   \
      int i_startidx_in, int i_endidx_in, int slev, int elev, int nproma,      \
      bool lacc, bool acc_async, int nlev, int nblks_e, int nblks_v)

// Declare as templates
template <typename T> ICONMATH_DECLARE_RECON_LSQ_CELL_L(T);
template <typename T> ICONMATH_DECLARE_RECON_LSQ_CELL_L_SVD(T);
template <typename T> ICONMATH_DECLARE_RECON_LSQ_CELL_Q(T);
template <typename T> ICONMATH_DECLARE_RECON_LSQ_CELL_Q_SVD(T);
template <typename T> ICONMATH_DECLARE_RECON_LSQ_CELL_C(T);
template <typename T> ICONMATH_DECLARE_RECON_LSQ_CELL_C_SVD(T);
template <typename T> ICONMATH_DECLARE_DIV3D(T);
template <typename T> ICONMATH_DECLARE_DIV3D_2FIELD(T);
template <typename T> ICONMATH_DECLARE_DIV4D(T);
template <typename T> ICONMATH_DECLARE_DIV_AVG(T);
template <typename T> ICONMATH_DECLARE_ROT_VERTEX_ATMOS(T);
template <typename T> ICONMATH_DECLARE_ROT_VERTEX_RI(T);
