// ICON
//
// ---------------------------------------------------------------
// Copyright (C) 2004-2025, DWD, MPI-M, DKRZ, KIT, ETH, MeteoSwiss
// Contact information: icon-model.org
//
// See AUTHORS.TXT for a list of authors
// See LICENSES/ for license information
// SPDX-License-Identifier: BSD-3-Clause
// ---------------------------------------------------------------

#include "interpolation_bindings.h"
#include "mo_lib_interpolation_scalar.hpp"
#include "mo_lib_interpolation_vector.hpp"
#include "mo_lib_intp_rbf.hpp"

// This is the binding for mo_interpolation_vector::edges2cells_vector_lib
// (wp=dp)
void edges2cells_vector_lib_dp(const double *p_vn_in, const double *p_vt_in,
                               const int *cell_edge_idx,
                               const int *cell_edge_blk,
                               const double *e_bln_c_u, const double *e_bln_c_v,
                               double *p_u_out, double *p_v_out, int i_startblk,
                               int i_endblk, int i_startidx_in, int i_endidx_in,
                               int slev, int elev, int nproma, int nlev,
                               int nblks_e, int nblks_c) {

  edges2cells_vector_lib<double>(
      p_vn_in, p_vt_in, cell_edge_idx, cell_edge_blk, e_bln_c_u, e_bln_c_v,
      p_u_out, p_v_out, i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev,
      elev, nproma, nlev, nblks_e, nblks_c);
}

// This is the binding for mo_interpolation_vector::edges2cells_vector_lib
// (wp=sp)
void edges2cells_vector_lib_sp(const float *p_vn_in, const float *p_vt_in,
                               const int *cell_edge_idx,
                               const int *cell_edge_blk, const float *e_bln_c_u,
                               const float *e_bln_c_v, float *p_u_out,
                               float *p_v_out, int i_startblk, int i_endblk,
                               int i_startidx_in, int i_endidx_in, int slev,
                               int elev, int nproma, int nlev, int nblks_e,
                               int nblks_c) {

  edges2cells_vector_lib<float>(
      p_vn_in, p_vt_in, cell_edge_idx, cell_edge_blk, e_bln_c_u, e_bln_c_v,
      p_u_out, p_v_out, i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev,
      elev, nproma, nlev, nblks_e, nblks_c);
}

// This is the binding for mo_interpolation_scalar::verts2edges_scalar_lib
// (wp=dp)
void verts2edges_scalar_lib_dp(
    const double *p_vertex_in, const int *edge_vertex_idx,
    const int *edge_vertex_blk, const double *coeff_int, double *p_edge_out,
    const int i_startblk, const int i_endblk, const int i_startidx_in,
    const int i_endidx_in, const int slev, const int elev, const int nproma,
    const int nlev, const int nblks_v, const int nblks_e, const bool lacc) {

  verts2edges_scalar_lib<double>(p_vertex_in, edge_vertex_idx, edge_vertex_blk,
                                 coeff_int, p_edge_out, i_startblk, i_endblk,
                                 i_startidx_in, i_endidx_in, slev, elev, nproma,
                                 nlev, nblks_v, nblks_e, lacc);
}

// This is the binding for mo_interpolation_scalar::verts2edges_scalar_lib
// (wp=sp)
void verts2edges_scalar_lib_sp(
    const float *p_vertex_in, const int *edge_vertex_idx,
    const int *edge_vertex_blk, const float *coeff_int, float *p_edge_out,
    const int i_startblk, const int i_endblk, const int i_startidx_in,
    const int i_endidx_in, const int slev, const int elev, const int nproma,
    const int nlev, const int nblks_v, const int nblks_e, const bool lacc) {

  verts2edges_scalar_lib<float>(p_vertex_in, edge_vertex_idx, edge_vertex_blk,
                                coeff_int, p_edge_out, i_startblk, i_endblk,
                                i_startidx_in, i_endidx_in, slev, elev, nproma,
                                nlev, nblks_v, nblks_e, lacc);
}

// This is the binding for mo_interpolation_scalar::cells2edges_scalar_dp_lib
void cells2edges_scalar_lib_dp(
    const double *p_cell_in, const int *edge_cell_idx, const int *edge_cell_blk,
    const double *coeff_int, double *p_edge_out, const int *i_startblk_in,
    const int *i_endblk_in, const int *i_startidx_in, const int *i_endidx_in,
    const int slev, const int elev, const int nproma, const int nlev,
    const int nblk_c, const int nblks_e, const int patch_id,
    const bool l_limited_area, const bool lfill_latbc, const bool lacc) {

  cells2edges_scalar_lib<double, double>(
      p_cell_in, edge_cell_idx, edge_cell_blk, coeff_int, p_edge_out,
      i_startblk_in, i_endblk_in, i_startidx_in, i_endidx_in, slev, elev,
      nproma, nlev, nblk_c, nblks_e, patch_id, l_limited_area, lfill_latbc,
      lacc);
}

// This is the binding for mo_interpolation_scalar::cells2edges_scalar_sp_lib
void cells2edges_scalar_lib_sp(const float *p_cell_in, const int *edge_cell_idx,
                               const int *edge_cell_blk, const float *coeff_int,
                               float *p_edge_out, const int *i_startblk_in,
                               const int *i_endblk_in, const int *i_startidx_in,
                               const int *i_endidx_in, const int slev,
                               const int elev, const int nproma, const int nlev,
                               const int nblk_c, const int nblks_e,
                               const int patch_id, const bool l_limited_area,
                               const bool lfill_latbc, const bool lacc) {

  cells2edges_scalar_lib<float, float>(
      p_cell_in, edge_cell_idx, edge_cell_blk, coeff_int, p_edge_out,
      i_startblk_in, i_endblk_in, i_startidx_in, i_endidx_in, slev, elev,
      nproma, nlev, nblk_c, nblks_e, patch_id, l_limited_area, lfill_latbc,
      lacc);
}

// This is the binding for mo_interpolation_scalar::cells2edges_scalar_sp2dp_lib
void cells2edges_scalar_lib_sp2dp(
    const float *p_cell_in, const int *edge_cell_idx, const int *edge_cell_blk,
    const double *coeff_int, double *p_edge_out, const int *i_startblk_in,
    const int *i_endblk_in, const int *i_startidx_in, const int *i_endidx_in,
    const int slev, const int elev, const int nproma, const int nlev,
    const int nblk_c, const int nblks_e, const int patch_id,
    const bool l_limited_area, const bool lfill_latbc, const bool lacc) {

  cells2edges_scalar_lib<float, double>(
      p_cell_in, edge_cell_idx, edge_cell_blk, coeff_int, p_edge_out,
      i_startblk_in, i_endblk_in, i_startidx_in, i_endidx_in, slev, elev,
      nproma, nlev, nblk_c, nblks_e, patch_id, l_limited_area, lfill_latbc,
      lacc);
}

// This is the binding for mo_interpolation_scalar::edges2verts_scalar_lib
// (wp=dp)
void edges2verts_scalar_lib_dp(
    const double *p_edge_in, const int *vert_edge_idx, const int *vert_edge_blk,
    const double *v_int, double *p_vert_out, const int i_startblk,
    const int i_endblk, const int i_startidx_in, const int i_endidx_in,
    const int slev, const int elev, const int nproma, const int nlev,
    const int nblks_e, const int nblks_v, const bool lacc) {

  edges2verts_scalar_lib<double>(p_edge_in, vert_edge_idx, vert_edge_blk, v_int,
                                 p_vert_out, i_startblk, i_endblk,
                                 i_startidx_in, i_endidx_in, slev, elev, nproma,
                                 nlev, nblks_e, nblks_v, lacc);
}

// This is the binding for mo_interpolation_scalar::edges2verts_scalar_lib
// (wp=sp)
void edges2verts_scalar_lib_sp(const float *p_edge_in, const int *vert_edge_idx,
                               const int *vert_edge_blk, const float *v_int,
                               float *p_vert_out, const int i_startblk,
                               const int i_endblk, const int i_startidx_in,
                               const int i_endidx_in, const int slev,
                               const int elev, const int nproma, const int nlev,
                               const int nblks_e, const int nblks_v,
                               const bool lacc) {

  edges2verts_scalar_lib<float>(p_edge_in, vert_edge_idx, vert_edge_blk, v_int,
                                p_vert_out, i_startblk, i_endblk, i_startidx_in,
                                i_endidx_in, slev, elev, nproma, nlev, nblks_e,
                                nblks_v, lacc);
}

// This is the binding for mo_interpolation_scalar::edges2cells_scalar_dp_lib
void edges2cells_scalar_lib_dp(const double *p_edge_in, const int *edge_idx,
                               const int *edge_blk, const double *coeff_int,
                               double *p_cell_out, const int i_startblk,
                               const int i_endblk, const int i_startidx_in,
                               const int i_endidx_in, const int slev,
                               const int elev, const int nproma, const int nlev,
                               const int nblks_e, const int nblks_c,
                               const bool lacc) {
  edges2cells_scalar_lib<double>(p_edge_in, edge_idx, edge_blk, coeff_int,
                                 p_cell_out, i_startblk, i_endblk,
                                 i_startidx_in, i_endidx_in, slev, elev, nproma,
                                 nlev, nblks_e, nblks_c, lacc);
}

// This is the binding for mo_interpolation_scalar::edges2cells_scalar_sp_lib
void edges2cells_scalar_lib_sp(const float *p_edge_in, const int *edge_idx,
                               const int *edge_blk, const float *coeff_int,
                               float *p_cell_out, const int i_startblk,
                               const int i_endblk, const int i_startidx_in,
                               const int i_endidx_in, const int slev,
                               const int elev, const int nproma, const int nlev,
                               const int nblks_e, const int nblks_c,
                               const bool lacc) {

  edges2cells_scalar_lib<float>(p_edge_in, edge_idx, edge_blk, coeff_int,
                                p_cell_out, i_startblk, i_endblk, i_startidx_in,
                                i_endidx_in, slev, elev, nproma, nlev, nblks_e,
                                nblks_c, lacc);
}

// This is the binding for mo_interpolation_scalar::cells2verts_scalar_dp_lib
void cells2verts_scalar_lib_dp(
    const double *p_cell_in, const int *vert_cell_idx, const int *vert_cell_blk,
    const double *coeff_int, double *p_vert_out, const int i_startblk,
    const int i_endblk, const int i_startidx_in, const int i_endidx_in,
    const int slev, const int elev, const int nproma, const int nlev,
    const int nblks_c, const int nblks_v, const bool lacc,
    const bool acc_async) {
  cells2verts_scalar_lib<double, double>(
      p_cell_in, vert_cell_idx, vert_cell_blk, coeff_int, p_vert_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      nlev, nblks_c, nblks_v, lacc, acc_async);
}

// This is the binding for mo_interpolation_scalar::cells2verts_scalar_dp2sp_lib
void cells2verts_scalar_lib_dp2sp(
    const float *p_cell_in, const int *vert_cell_idx, const int *vert_cell_blk,
    const double *coeff_int, double *p_vert_out, const int i_startblk,
    const int i_endblk, const int i_startidx_in, const int i_endidx_in,
    const int slev, const int elev, const int nproma, const int nlev,
    const int nblks_c, const int nblks_v, const bool lacc,
    const bool acc_async) {
  cells2verts_scalar_lib<float, double>(
      p_cell_in, vert_cell_idx, vert_cell_blk, coeff_int, p_vert_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      nlev, nblks_c, nblks_v, lacc, acc_async);
}

// This is the binding for mo_interpolation_scalar::cells2verts_scalar_sp_lib
void cells2verts_scalar_lib_sp(const float *p_cell_in, const int *vert_cell_idx,
                               const int *vert_cell_blk, const float *coeff_int,
                               float *p_vert_out, const int i_startblk,
                               const int i_endblk, const int i_startidx_in,
                               const int i_endidx_in, const int slev,
                               const int elev, const int nproma, const int nlev,
                               const int nblks_c, const int nblks_v,
                               const bool lacc, const bool acc_async) {
  cells2verts_scalar_lib<float, float>(
      p_cell_in, vert_cell_idx, vert_cell_blk, coeff_int, p_vert_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      nlev, nblks_c, nblks_v, lacc, acc_async);
}

// This is the binding for mo_interpolation_scalar::cells2verts_scalar_ri_lib
// (wp=dp, vp=dp)
void cells2verts_scalar_ri_lib_dp(
    const double *p_cell_in, const int *vert_cell_idx, const int *vert_cell_blk,
    const double *coeff_int, double *p_vert_out, const int i_startblk,
    const int i_endblk, const int i_startidx_in, const int i_endidx_in,
    const int slev, const int elev, const int nproma, const int nlev,
    const int nblks_c, const int nblks_v, const bool lacc,
    const bool acc_async) {
  cells2verts_scalar_ri_lib<double, double>(
      p_cell_in, vert_cell_idx, vert_cell_blk, coeff_int, p_vert_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      nlev, nblks_c, nblks_v, lacc, acc_async);
}

// This is the binding for mo_interpolation_scalar::cells2verts_scalar_ri_lib
// (wp=dp, vp=sp)
void cells2verts_scalar_ri_lib_dp2sp(
    const double *p_cell_in, const int *vert_cell_idx, const int *vert_cell_blk,
    const double *coeff_int, float *p_vert_out, const int i_startblk,
    const int i_endblk, const int i_startidx_in, const int i_endidx_in,
    const int slev, const int elev, const int nproma, const int nlev,
    const int nblks_c, const int nblks_v, const bool lacc,
    const bool acc_async) {
  cells2verts_scalar_ri_lib<double, float>(
      p_cell_in, vert_cell_idx, vert_cell_blk, coeff_int, p_vert_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      nlev, nblks_c, nblks_v, lacc, acc_async);
}

// This is the binding for mo_interpolation_scalar::cells2verts_scalar_ri_lib
// (wp=sp, vp=sp)
void cells2verts_scalar_ri_lib_sp(
    const float *p_cell_in, const int *vert_cell_idx, const int *vert_cell_blk,
    const float *coeff_int, float *p_vert_out, const int i_startblk,
    const int i_endblk, const int i_startidx_in, const int i_endidx_in,
    const int slev, const int elev, const int nproma, const int nlev,
    const int nblks_c, const int nblks_v, const bool lacc,
    const bool acc_async) {
  cells2verts_scalar_ri_lib<float, float>(
      p_cell_in, vert_cell_idx, vert_cell_blk, coeff_int, p_vert_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      nlev, nblks_c, nblks_v, lacc, acc_async);
}

// This is the binding for mo_interpolation_scalar::verts2cells_scalar_lib
// (wp=dp)
void verts2cells_scalar_lib_dp(
    const double *p_vert_in, const int *cell_index_idx,
    const int *cell_vertex_blk, const double *coeff_int, double *p_cell_out,
    const int nblks_c, const int npromz_c, const int slev, const int elev,
    const int nproma, const int nlev, const int nblks_v, const bool lacc) {
  verts2cells_scalar_lib<double>(p_vert_in, cell_index_idx, cell_vertex_blk,
                                 coeff_int, p_cell_out, nblks_c, npromz_c, slev,
                                 elev, nproma, nlev, nblks_v, lacc);
}

// This is the binding for mo_interpolation_scalar::verts2cells_scalar_lib
// (wp=sp)
void verts2cells_scalar_lib_sp(
    const float *p_vert_in, const int *cell_index_idx,
    const int *cell_vertex_blk, const float *coeff_int, float *p_cell_out,
    const int nblks_c, const int npromz_c, const int slev, const int elev,
    const int nproma, const int nlev, const int nblks_v, const bool lacc) {
  verts2cells_scalar_lib<float>(p_vert_in, cell_index_idx, cell_vertex_blk,
                                coeff_int, p_cell_out, nblks_c, npromz_c, slev,
                                elev, nproma, nlev, nblks_v, lacc);
}

// This is the binding for mo_interpolation_scalar::cell_avg_lib (wp=dp)
void cell_avg_lib_dp(const double *psi_c, const int *cell_neighbor_idx,
                     const int *cell_neighbor_blk, const double *avg_coeff,
                     double *avg_psi_c, const int i_startblk,
                     const int i_endblk, const int i_startidx_in,
                     const int i_endidx_in, const int slev, const int elev,
                     const int nproma, const int nlev, const int nblks_c,
                     const bool lacc) {
  cell_avg_lib<double>(psi_c, cell_neighbor_idx, cell_neighbor_blk, avg_coeff,
                       avg_psi_c, i_startblk, i_endblk, i_startidx_in,
                       i_endidx_in, slev, elev, nproma, nlev, nblks_c, lacc);
}

// This is the binding for mo_interpolation_scalar::cell_avg_lib (wp=sp)
void cell_avg_lib_sp(const float *psi_c, const int *cell_neighbor_idx,
                     const int *cell_neighbor_blk, const float *avg_coeff,
                     float *avg_psi_c, const int i_startblk, const int i_endblk,
                     const int i_startidx_in, const int i_endidx_in,
                     const int slev, const int elev, const int nproma,
                     const int nlev, const int nblks_c, const bool lacc) {
  cell_avg_lib<float>(psi_c, cell_neighbor_idx, cell_neighbor_blk, avg_coeff,
                      avg_psi_c, i_startblk, i_endblk, i_startidx_in,
                      i_endidx_in, slev, elev, nproma, nlev, nblks_c, lacc);
}

// This is the binding for mo_intp_rbf::rbf_vec_interpol_vertex_dp_lib
void rbf_vec_interpol_vertex_lib_dp(
    const double *p_e_in, const int *rbf_vec_idx_v, const int *rbf_vec_blk_v,
    const double *rbf_vec_coeff_v, double *p_u_out, double *p_v_out,
    const int i_startblk, const int i_endblk, const int i_startidx_in,
    const int i_endidx_in, const int slev, const int elev, const int nproma,
    const bool lacc, const bool acc_async, const int nlev, const int nblks_e,
    const int nblks_v) {
  rbf_vec_interpol_vertex_lib<double, double>(
      p_e_in, rbf_vec_idx_v, rbf_vec_blk_v, rbf_vec_coeff_v, p_u_out, p_v_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      lacc, acc_async, nlev, nblks_e, nblks_v);
}

// This is the binding for mo_intp_rbf::rbf_vec_interpol_vertex_sp_lib
void rbf_vec_interpol_vertex_lib_sp(
    const float *p_e_in, const int *rbf_vec_idx_v, const int *rbf_vec_blk_v,
    const float *rbf_vec_coeff_v, float *p_u_out, float *p_v_out,
    const int i_startblk, const int i_endblk, const int i_startidx_in,
    const int i_endidx_in, const int slev, const int elev, const int nproma,
    const bool lacc, const bool acc_async, const int nlev, const int nblks_e,
    const int nblks_v) {
  rbf_vec_interpol_vertex_lib<float, float>(
      p_e_in, rbf_vec_idx_v, rbf_vec_blk_v, rbf_vec_coeff_v, p_u_out, p_v_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      lacc, acc_async, nlev, nblks_e, nblks_v);
}

// This is the binding for mo_intp_rbf::rbf_vec_interpol_vertex_dpsp_lib
void rbf_vec_interpol_vertex_lib_dpsp(
    const double *p_e_in, const int *rbf_vec_idx_v, const int *rbf_vec_blk_v,
    const double *rbf_vec_coeff_v, float *p_u_out, float *p_v_out,
    const int i_startblk, const int i_endblk, const int i_startidx_in,
    const int i_endidx_in, const int slev, const int elev, const int nproma,
    const bool lacc, const bool acc_async, const int nlev, const int nblks_e,
    const int nblks_v) {
  rbf_vec_interpol_vertex_lib<double, float>(
      p_e_in, rbf_vec_idx_v, rbf_vec_blk_v, rbf_vec_coeff_v, p_u_out, p_v_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      lacc, acc_async, nlev, nblks_e, nblks_v);
}

// This is the binding for mo_intp_rbf::rbf_vec_interpol_cell_lib (wp=dp)
void rbf_interpol_c2grad_lib_sp(const float *p_cell_in,
                                const int *rbf_c2grad_idx,
                                const int *rbf_c2grad_blk,
                                const float *rbf_c2grad_coeff, float *grad_x,
                                float *grad_y, int i_startblk, int i_endblk,
                                int i_startidx_in, int i_endidx_in, int slev,
                                int elev, int nproma, int rbf_c2grad_dim,
                                int nlev, int nblk_c, bool lacc) {

  rbf_interpol_c2grad_lib<float>(
      p_cell_in, rbf_c2grad_idx, rbf_c2grad_blk, rbf_c2grad_coeff, grad_x,
      grad_y, i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev,
      nproma, rbf_c2grad_dim, nlev, nblk_c, lacc);
}

// This is the binding for mo_intp_rbf::rbf_vec_interpol_cell_lib (wp=sp)
void rbf_interpol_c2grad_lib_dp(const double *p_cell_in,
                                const int *rbf_c2grad_idx,
                                const int *rbf_c2grad_blk,
                                const double *rbf_c2grad_coeff, double *grad_x,
                                double *grad_y, int i_startblk, int i_endblk,
                                int i_startidx_in, int i_endidx_in, int slev,
                                int elev, int nproma, int rbf_c2grad_dim,
                                int nlev, int nblk_c, bool lacc) {

  rbf_interpol_c2grad_lib<double>(
      p_cell_in, rbf_c2grad_idx, rbf_c2grad_blk, rbf_c2grad_coeff, grad_x,
      grad_y, i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev,
      nproma, rbf_c2grad_dim, nlev, nblk_c, lacc);
}

// This is the binding for mo_intp_rbf::rbf_vec_interpol_cell_lib (wp=dp)
void rbf_vec_interpol_cell_lib_sp(
    const float *p_vn_in, const int *rbf_vec_idx_c, const int *rbf_vec_blk_c,
    const float *rbf_vec_coeff_c, float *p_u_out, float *p_v_out,
    int i_startblk, int i_endblk, int i_startidx_in, int i_endidx_in, int slev,
    int elev, int nproma, int nlev, int nblks_c, int nblks_e, int rbf_vec_dim_c,
    bool lacc, bool acc_async) {

  rbf_vec_interpol_cell_lib<float>(
      p_vn_in, rbf_vec_idx_c, rbf_vec_blk_c, rbf_vec_coeff_c, p_u_out, p_v_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      nlev, nblks_c, nblks_e, rbf_vec_dim_c, lacc, acc_async);
}

// This is the binding for mo_intp_rbf::rbf_vec_interpol_cell_lib (wp=sp)
void rbf_vec_interpol_cell_lib_dp(
    const double *p_vn_in, const int *rbf_vec_idx_c, const int *rbf_vec_blk_c,
    const double *rbf_vec_coeff_c, double *p_u_out, double *p_v_out,
    int i_startblk, int i_endblk, int i_startidx_in, int i_endidx_in, int slev,
    int elev, int nproma, int nlev, int nblks_c, int nblks_e, int rbf_vec_dim_c,
    bool lacc, bool acc_async) {

  rbf_vec_interpol_cell_lib<double>(
      p_vn_in, rbf_vec_idx_c, rbf_vec_blk_c, rbf_vec_coeff_c, p_u_out, p_v_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nproma,
      nlev, nblks_c, nblks_e, rbf_vec_dim_c, lacc, acc_async);
}

// This is the binding for mo_intp_rbf::rbf_vec_interpol_edge_lib (wp=dp)
void rbf_vec_interpol_edge_lib_dp(
    const double *p_vn_in, const int *rbf_vec_idx_e, const int *rbf_vec_blk_e,
    const double *rbf_vec_coeff_e, double *p_vt_out, int i_startblk,
    int i_endblk, int i_startidx_in, int i_endidx_in, int slev, int elev,
    int nlev, int nproma, int rbf_vec_dim_e, int nblks_e, bool lacc,
    bool acc_async) {

  rbf_vec_interpol_edge_lib<double>(
      p_vn_in, rbf_vec_idx_e, rbf_vec_blk_e, rbf_vec_coeff_e, p_vt_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nlev,
      nproma, rbf_vec_dim_e, nblks_e, lacc, acc_async);
}

// This is the binding for mo_intp_rbf::rbf_vec_interpol_edge_lib (wp=sp)
void rbf_vec_interpol_edge_lib_sp(
    const float *p_vn_in, const int *rbf_vec_idx_e, const int *rbf_vec_blk_e,
    const float *rbf_vec_coeff_e, float *p_vt_out, int i_startblk, int i_endblk,
    int i_startidx_in, int i_endidx_in, int slev, int elev, int nlev,
    int nproma, int rbf_vec_dim_e, int nblks_e, bool lacc, bool acc_async) {

  rbf_vec_interpol_edge_lib<float>(
      p_vn_in, rbf_vec_idx_e, rbf_vec_blk_e, rbf_vec_coeff_e, p_vt_out,
      i_startblk, i_endblk, i_startidx_in, i_endidx_in, slev, elev, nlev,
      nproma, rbf_vec_dim_e, nblks_e, lacc, acc_async);
}
