// ICON
//
// ---------------------------------------------------------------
// Copyright (C) 2004-2025, DWD, MPI-M, DKRZ, KIT, ETH, MeteoSwiss
// Contact information: icon-model.org
//
// See AUTHORS.TXT for a list of authors
// See LICENSES/ for license information
// SPDX-License-Identifier: BSD-3-Clause
// ---------------------------------------------------------------

#pragma once

template <typename T>
void verts2edges_scalar_lib(const T *p_vertex_in, const int *edge_vertex_idx,
                            const int *edge_vertex_blk, const T *coeff_int,
                            T *p_edge_out, const int i_startblk,
                            const int i_endblk, const int i_startidx_in,
                            const int i_endidx_in, const int slev,
                            const int elev, const int nproma, const int nlev,
                            const int nblks_v, const int nblks_e,
                            const bool lacc);
;

template <typename T, typename S>
void cells2edges_scalar_lib(const T *p_cell_in, const int *edge_cell_idx,
                            const int *edge_cell_blk, const S *coeff_int,
                            S *p_edge_out, const int *i_startblk_in,
                            const int *i_endblk_in, const int *i_startidx_in,
                            const int *i_endidx_in, const int slev,
                            const int elev, const int nproma, const int nlev,
                            const int nblk_c, const int nblks_e,
                            const int patch_id, const bool l_limited_area,
                            const bool lfill_latbc, const bool lacc);

template <typename T>
void edges2verts_scalar_lib(const T *p_edge_in, const int *vert_edge_idx,
                            const int *vert_edge_blk, const T *v_int,
                            T *p_vert_out, const int i_startblk,
                            const int i_endblk, const int i_startidx_in,
                            const int i_endidx_in, const int slev,
                            const int elev, const int nproma, const int nlev,
                            const int nblks_e, const int nblks_v,
                            const bool lacc);

template <typename T>
void edges2cells_scalar_lib(const T *p_edge_in, const int *edge_idx,
                            const int *edge_blk, const T *coeff_int,
                            T *p_cell_out, const int i_startblk,
                            const int i_endblk, const int i_startidx_in,
                            const int i_endidx_in, const int slev,
                            const int elev, const int nproma, const int nlev,
                            const int nblks_e, const int nblks_c,
                            const bool lacc);

template <typename T, typename S>
void cells2verts_scalar_lib(const T *p_cell_in, const int *vert_cell_idx,
                            const int *vert_cell_blk, const S *coeff_int,
                            S *p_vert_out, const int i_startblk,
                            const int i_endblk, const int i_startidx_in,
                            const int i_endidx_in, const int slev,
                            const int elev, const int nproma, const int nlev,
                            const int nblks_c, const int nblks_v,
                            const bool lacc, const bool acc_async);

template <typename T, typename S>
void cells2verts_scalar_ri_lib(const T *p_cell_in, const int *vert_cell_idx,
                               const int *vert_cell_blk, const T *coeff_int,
                               S *p_vert_out, const int i_startblk,
                               const int i_endblk, const int i_startidx_in,
                               const int i_endidx_in, const int slev,
                               const int elev, const int nproma, const int nlev,
                               const int nblks_c, const int nblks_v,
                               const bool lacc, const bool acc_async);

template <typename T>
void verts2cells_scalar_lib(const T *p_vert_in, const int *cell_index_idx,
                            const int *cell_vertex_blk, const T *coeff_int,
                            T *p_cell_out, const int nblks_c,
                            const int npromz_c, const int slev, const int elev,
                            const int nproma, const int nlev, const int nblks_v,
                            const bool lacc);

template <typename T>
void cell_avg_lib(const T *psi_c, const int *cell_neighbor_idx,
                  const int *cell_neighbor_blk, const T *avg_coeff,
                  T *avg_psi_c, const int i_startblk, const int i_endblk,
                  const int i_startidx_in, const int i_endidx_in,
                  const int slev, const int elev, const int nproma,
                  const int nlev, const int nblks_c, const bool lacc);
