// ICON
//
// ---------------------------------------------------------------
// Copyright (C) 2004-2025, DWD, MPI-M, DKRZ, KIT, ETH, MeteoSwiss
// Contact information: icon-model.org
//
// See AUTHORS.TXT for a list of authors
// See LICENSES/ for license information
// SPDX-License-Identifier: BSD-3-Clause
// ---------------------------------------------------------------

#include <algorithm> // For std::max

// get_indices_c_lib function
void get_indices_c_lib(const int i_startidx_in, const int i_endidx_in, const int nproma, 
                        const int i_blk, const int i_startblk, const int i_endblk,
                        int &i_startidx_out, int &i_endidx_out, const bool called_from_cpp=true) {
    
    //Since code is ported incrementally from Fortran to C++, depending on where the function is called from
    //(either fortran or c++), the first index should be either 0 or 1.
    int first_index;
    if (called_from_cpp)
        first_index = 0;
    else
        first_index = 1;                   
    
    if (i_blk == i_startblk) {
        i_startidx_out = std::max(first_index, i_startidx_in);
        i_endidx_out = nproma;
        if (i_blk == i_endblk) {
            i_endidx_out = i_endidx_in;
        }
    } else if (i_blk == i_endblk) {
        i_startidx_out = first_index;
        i_endidx_out = i_endidx_in;
    } else {
        i_startidx_out = first_index;
        i_endidx_out = nproma;
    }
}

// get_indices_e_lib function
void get_indices_e_lib(const int i_startidx_in, const int i_endidx_in, const int nproma, 
                        const int i_blk, const int i_startblk, const int i_endblk,
                        int &i_startidx_out, int &i_endidx_out, const bool called_from_cpp=true) {
    
    //Since code is ported incrementally from Fortran to C++, depending on where the function is called from, 
    //the first index should be either 0 or 1.
    int first_index;
    if (called_from_cpp)
        first_index = 0;
    else
        first_index = 1;

    i_startidx_out = (i_blk != i_startblk) ? first_index : std::max(first_index, i_startidx_in);
    i_endidx_out = (i_blk != i_endblk) ? nproma : i_endidx_in;
}

// get_indices_v_lib function
void get_indices_v_lib(const int i_startidx_in, const int i_endidx_in, const int nproma, 
                        const int i_blk, const int i_startblk, const int i_endblk,
                        int &i_startidx_out, int &i_endidx_out, const bool called_from_cpp=true) {
    
    //Since code is ported incrementally from Fortran to C++, depending on where the function is called from, 
    //the first index should be either 0 or 1.
    int first_index;
    if (called_from_cpp)
        first_index = 0;
    else
        first_index = 1;

    if (i_blk == i_startblk) {
        i_startidx_out = i_startidx_in;
        i_endidx_out = nproma;
        if (i_blk == i_endblk) {
            i_endidx_out = i_endidx_in;
        }
    } else if (i_blk == i_endblk) {
        i_startidx_out = first_index;
        i_endidx_out = i_endidx_in;
    } else {
        i_startidx_out = first_index;
        i_endidx_out = nproma;
    }
}