! ICON
!
! ---------------------------------------------------------------
! Copyright (C) 2004-2025, DWD, MPI-M, DKRZ, KIT, ETH, MeteoSwiss
! Contact information: icon-model.org
!
! See AUTHORS.TXT for a list of authors
! See LICENSES/ for license information
! SPDX-License-Identifier: BSD-3-Clause
! ---------------------------------------------------------------

MODULE TEST_mo_lib_loopindices
  USE FORTUTF

  IMPLICIT NONE

  PRIVATE

  PUBLIC :: TEST_get_indices_lib

CONTAINS

  SUBROUTINE TEST_get_indices_lib
    USE mo_lib_loopindices, ONLY: get_indices_c_lib, get_indices_e_lib, get_indices_v_lib

    INTEGER :: i_startidx_in ! Start index as input
    INTEGER :: i_endidx_in ! End index as input
    INTEGER :: nproma ! inner loop length/vector length
    INTEGER :: i_blk ! Current block (variable jb in do loops)
    INTEGER :: i_startblk ! Start block of do loop
    INTEGER :: i_endblk ! End block of do loop

    INTEGER :: i_startidx_out, i_endidx_out ! Start and end indices (jc loop), as output

    i_startidx_in = 2
    i_endidx_in = 15
    nproma = 32
    i_startblk = 1
    i_endblk = 40

    ! CASE: I -> i_blk == i_startblk
    i_blk = 1
    CALL get_indices_c_lib(i_startidx_in, i_endidx_in, nproma, i_blk, i_startblk, i_endblk, &
                           i_startidx_out, i_endidx_out)

    CALL TAG_TEST("TEST_get_indices_c_start_1")
    CALL ASSERT_EQUAL(i_startidx_out, MAX(1, i_startidx_in))
    CALL TAG_TEST("TEST_get_indices_c_end_1")
    CALL ASSERT_EQUAL(i_endidx_out, nproma)

    i_startidx_out = 0
    i_endidx_out = 0

    CALL get_indices_e_lib(i_startidx_in, i_endidx_in, nproma, i_blk, i_startblk, i_endblk, &
                           i_startidx_out, i_endidx_out)

    CALL TAG_TEST("TEST_get_indices_e_start_1")
    CALL ASSERT_EQUAL(i_startidx_out, MAX(1, i_startidx_in))
    CALL TAG_TEST("TEST_get_indices_e_end_1")
    CALL ASSERT_EQUAL(i_endidx_out, nproma)

    i_startidx_out = 0
    i_endidx_out = 0

    CALL get_indices_v_lib(i_startidx_in, i_endidx_in, nproma, i_blk, i_startblk, i_endblk, &
                           i_startidx_out, i_endidx_out)

    CALL TAG_TEST("TEST_get_indices_v_start_1")
    CALL ASSERT_EQUAL(i_startidx_out, MAX(1, i_startidx_in))
    CALL TAG_TEST("TEST_get_indices_v_end_1")
    CALL ASSERT_EQUAL(i_endidx_out, nproma)

    i_startidx_out = 0
    i_endidx_out = 0

    ! CASE: II -> i_blk == i_endblk
    i_blk = 40
    CALL get_indices_c_lib(i_startidx_in, i_endidx_in, nproma, i_blk, i_startblk, i_endblk, &
                           i_startidx_out, i_endidx_out)

    CALL TAG_TEST("TEST_get_indices_c_start_2")
    CALL ASSERT_EQUAL(i_startidx_out, 1)
    CALL TAG_TEST("TEST_get_indices_c_end_2")
    CALL ASSERT_EQUAL(i_endidx_out, i_endidx_in)

    i_startidx_out = 0
    i_endidx_out = 0

    CALL get_indices_e_lib(i_startidx_in, i_endidx_in, nproma, i_blk, i_startblk, i_endblk, &
                           i_startidx_out, i_endidx_out)

    CALL TAG_TEST("TEST_get_indices_e_start_2")
    CALL ASSERT_EQUAL(i_startidx_out, 1)
    CALL TAG_TEST("TEST_get_indices_e_end_2")
    CALL ASSERT_EQUAL(i_endidx_out, i_endidx_in)

    i_startidx_out = 0
    i_endidx_out = 0

    CALL get_indices_v_lib(i_startidx_in, i_endidx_in, nproma, i_blk, i_startblk, i_endblk, &
                           i_startidx_out, i_endidx_out)

    CALL TAG_TEST("TEST_get_indices_v_start_2")
    CALL ASSERT_EQUAL(i_startidx_out, 1)
    CALL TAG_TEST("TEST_get_indices_v_end_2")
    CALL ASSERT_EQUAL(i_endidx_out, i_endidx_in)

    i_startidx_out = 0
    i_endidx_out = 0

    ! CASE: III -> Every other cases
    i_blk = 20
    CALL get_indices_c_lib(i_startidx_in, i_endidx_in, nproma, i_blk, i_startblk, i_endblk, &
                           i_startidx_out, i_endidx_out)

    CALL TAG_TEST("TEST_get_indices_c_start_3")
    CALL ASSERT_EQUAL(i_startidx_out, 1)
    CALL TAG_TEST("TEST_get_indices_c_end_3")
    CALL ASSERT_EQUAL(i_endidx_out, nproma)

    i_startidx_out = 0
    i_endidx_out = 0

    CALL get_indices_e_lib(i_startidx_in, i_endidx_in, nproma, i_blk, i_startblk, i_endblk, &
                           i_startidx_out, i_endidx_out)

    CALL TAG_TEST("TEST_get_indices_e_start_3")
    CALL ASSERT_EQUAL(i_startidx_out, 1)
    CALL TAG_TEST("TEST_get_indices_e_end_3")
    CALL ASSERT_EQUAL(i_endidx_out, nproma)

    i_startidx_out = 0
    i_endidx_out = 0

    CALL get_indices_v_lib(i_startidx_in, i_endidx_in, nproma, i_blk, i_startblk, i_endblk, &
                           i_startidx_out, i_endidx_out)

    CALL TAG_TEST("TEST_get_indices_v_start_3")
    CALL ASSERT_EQUAL(i_startidx_out, 1)
    CALL TAG_TEST("TEST_get_indices_v_end_3")
    CALL ASSERT_EQUAL(i_endidx_out, nproma)

  END SUBROUTINE TEST_get_indices_lib

END MODULE TEST_mo_lib_loopindices
